import { parse, visit, } from 'graphql';
import { getAutocompleteSuggestions, getDiagnostics, getHoverInformation, } from 'graphql-language-service-interface';
import { getVariablesJSONSchema, getOperationASTFacts, } from 'graphql-language-service-utils';
import { defaultSchemaLoader } from './schemaLoader';
export class LanguageService {
    constructor({ parser, schemaLoader, schemaConfig, parseOptions, exteralFragmentDefinitions, }) {
        this._parser = parse;
        this._schemaLoader = defaultSchemaLoader;
        this._parseOptions = undefined;
        this._exteralFragmentDefinitionNodes = null;
        this._exteralFragmentDefinitionsString = null;
        this.getCompletion = async (_uri, documentText, position) => {
            const schema = await this.getSchema();
            if (!documentText || documentText.length < 1 || !schema) {
                return [];
            }
            return getAutocompleteSuggestions(schema, documentText, position, undefined, await this.getExternalFragmentDefinitions());
        };
        this.getDiagnostics = async (_uri, documentText, customRules) => {
            const schema = await this.getSchema();
            if (!documentText || documentText.length < 1 || !schema) {
                return [];
            }
            return getDiagnostics(documentText, schema, customRules);
        };
        this.getHover = async (_uri, documentText, position) => getHoverInformation((await this.getSchema()), documentText, position);
        this.getVariablesJSONSchema = async (_uri, documentText) => {
            const schema = await this.getSchema();
            if (schema && documentText.length > 3) {
                try {
                    const documentAST = await this.parse(documentText);
                    const operationFacts = getOperationASTFacts(documentAST, schema);
                    if (operationFacts && operationFacts.variableToType) {
                        return getVariablesJSONSchema(operationFacts.variableToType);
                    }
                }
                catch (err) { }
            }
            return null;
        };
        this._schemaConfig = schemaConfig;
        if (parser) {
            this._parser = parser;
        }
        if (schemaLoader) {
            this._schemaLoader = schemaLoader;
        }
        if (parseOptions) {
            this._parseOptions = parseOptions;
        }
        if (exteralFragmentDefinitions) {
            if (Array.isArray(exteralFragmentDefinitions)) {
                this._exteralFragmentDefinitionNodes = exteralFragmentDefinitions;
            }
            else {
                this._exteralFragmentDefinitionsString = exteralFragmentDefinitions;
            }
        }
    }
    get schema() {
        return this._schemaConfig.schema;
    }
    get schemaConfig() {
        return this._schemaConfig;
    }
    get introspectionJSON() {
        return this._schemaConfig.introspectionJSON;
    }
    async getSchema() {
        if (this.schema) {
            return this.schema;
        }
        return this.loadSchema();
    }
    async getExternalFragmentDefinitions() {
        if (!this._exteralFragmentDefinitionNodes &&
            this._exteralFragmentDefinitionsString) {
            const definitionNodes = [];
            try {
                visit(await this._parser(this._exteralFragmentDefinitionsString), {
                    FragmentDefinition(node) {
                        definitionNodes.push(node);
                    },
                });
            }
            catch (err) {
                throw Error(`Failed parsing exteralFragmentDefinitions string:\n${this._exteralFragmentDefinitionsString}`);
            }
            this._exteralFragmentDefinitionNodes = definitionNodes;
        }
        return this._exteralFragmentDefinitionNodes;
    }
    async setSchema(schema) {
        this._schemaConfig.schema = schema;
        await this.loadSchema();
    }
    async loadSchema() {
        const result = await this._schemaLoader(this._schemaConfig, this.parse);
        if (result?.schema) {
            this._schemaConfig.schema = result.schema;
            if (result.introspectionJSON && !result.introspectionJSONString) {
                this._schemaConfig.introspectionJSON = result.introspectionJSON;
                this._schemaConfig.introspectionJSONString = JSON.stringify(result.introspectionJSONString);
            }
            else if (result.introspectionJSONString) {
                this._schemaConfig.introspectionJSON = JSON.parse(result.introspectionJSONString);
                this._schemaConfig.introspectionJSONString =
                    result.introspectionJSONString;
            }
        }
        return this._schemaConfig.schema ?? null;
    }
    parse(text, options) {
        return this._parser(text, options || this._parseOptions);
    }
}
//# sourceMappingURL=LanguageService.js.map